;Bulk only transport interface routines
;
;
;sendCmdOut
;sendCmdIn
;sendReset
;initalizeMSD
initializeMSD:
	;Initializes a mass storage device using bulk-out transport
	;Returns C if failure
	U_CALL_INIT MSDCallBack     ;Inialize U_CALL and Callback RAM calls
	ld a,errNoDrv
   jp c,displayErr       ;If it returns C, that means USB8X isn't loaded on calc
   ld hl,USBDriverBuf
   U_CALL DriverInit           ;Initialize USB driver
	ld hl,DescBuf
   U_CALL AutoSetup            ;Configure USB device (yay for autosetup)
	ld a,errConfigure
	jp c,displayErr
   ld hl,DescBuf
	U_CALL GetClass             ;Get the class information for the attached device
	cp 8
	ld a,errWrongClass
	jp nz,displayErr
	ld a,b
	cp 6
	ld a,errWrongClass
	jp nz,displayErr
	ld a,c
	cp 50h
	ld a,errWrongClass
	jp nz,displayErr
	xor a
	ld (BOTState),a
	ret
sendCmdOut:
	;Sends a command block out and then sends data in the bulk-only transport protocol.
	;Input - HL points to a command block. DE is the number of bytes that will be sent.
	;Output - C if there was a problem at any stage.
	ld a,(hl)
	or a
	scf
	ret z		;The command block length cannot be 0. Return an error condition if so.
	push af
	push hl
	;Build command block wrapper (CBW)
	push de
	ld de,BulkBuf
	ld hl,sendCmd_CBW_Start
	ld bc,8
	ldir
	;dCBWDataTransferLength
	pop de
	pop hl
	push hl
	ld hl,BulkBuf+8
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	ld (hl),0
	inc hl
	ld (hl),0
	;bmCBWFlags
	inc hl
	ld (hl),0
	;Reserved/bCBWLUN
	inc hl
	ld (hl),0
	;Reserved/bCBWCBLength
	pop de
	pop af
	push de
	and 00011111b
	inc hl
	ld (hl),a
	;CBWCB
	inc hl
	ex de,hl
	pop hl
	inc hl
	ld c,a
	ld b,0
	ldir
	ld b,a
	ld a,16
	sub b
	or a
	jr z,sendCmdO_2
	ld b,a
sendCmdO_1:
	xor a
	ld (de),a
	inc de
	djnz sendCmdO_1
sendCmdO_2:
	;Send CBW!
	ld c,02
	ld b,31
	ld hl,BulkBuf
	xor a
	ld (BOTState),a
	U_CALL SendData
	ret c
	;Send data now!
	ld hl,(DataInAddr)
sendCmdOut_L:
	ld c,02
	ld b,64
	push hl
	U_CALL SendData
	pop hl
	ret c
	ld de,64
	add hl,de
	push hl
	ld hl,(DataInExpected)
	or a
	sbc hl,de
	ld (DataInExpected),hl
	pop hl
	jr nz,sendCmdOut_L
	ld b,1
	U_CALL ReqData
	ret
sendCmdIn:
	;Sends a command block out and receives data in the bulk-only transport protocol.
	;Input - HL points to a command block. DE is the expected number of bytes that
	;will be received
	;Output - C if there was a problem at any stage
	;Output - PacketData contains a received packet PACKET
	;The first byte of the command block is the length (1-16).
	;The remaining bytes are the command block itself.
	ld a,(hl)
	or a
	scf
	ret z		;The command block length cannot be 0. Return an error condition if so.
	push af
	push hl
	;Build command block wrapper (CBW)
	push de
	ld de,BulkBuf
	ld hl,sendCmd_CBW_Start
	ld bc,8
	ldir
	;dCBWDataTransferLength
	pop de
	pop hl
	push hl
	ld hl,BulkBuf+8
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	ld (hl),0
	inc hl
	ld (hl),0
	;bmCBWFlags
	inc hl
	ld (hl),80h
	;Reserved/bCBWLUN
	inc hl
	ld (hl),0
	;Reserved/bCBWCBLength
	pop de
	pop af
	push de
	and 00011111b
	inc hl
	ld (hl),a
	;CBWCB
	inc hl
	ex de,hl
	pop hl
	inc hl
	ld c,a
	ld b,0
	ldir
	ld b,a
	ld a,16
	sub b
	or a
	jr z,sendCmd_2
	ld b,a
sendCmd_1:
	xor a
	ld (de),a
	inc de
	djnz sendCmd_1
sendCmd_2:
	;Send CBW!
	ld c,02
	ld b,31
	ld hl,BulkBuf
	;
	;ld de,savesscreen
	;ld bc,768
	;ldir
	;jp AppExit
	;
	ld a,1
	ld (BOTState),a
	U_CALL SendData
	ret c
	ld b,1
	U_CALL ReqData
	ld hl,0
	ld de,0
sendCmdIn_3:
	ld a,(BOTState)
	or a
	ret z
	ld bc,1
	add hl,bc
	ex de,hl
	ld bc,0
	adc hl,bc
	ex de,hl
	jr nc,sendCmdIn_3
	ret
sendCmd_CBW_Start:
	DB 55h,53h,42h,43h
	DB "3.14"
sendReset:
	;Send bulk reset command
	;Returns C if problem
	ld hl,bulk_reset
	ld de,8000h
	ld bc,8
	ldir
	ld hl,8000h
	ld b,0
	U_CALL SendControlData
	ret
bulk_reset:
	DB 21h,255,0,0,0,0,0,0
MSDCallBack:
	ld a,e
	;push de
	;call DispHexA
	;pop de
	ld a,(BOTState)
	dec a		;if 1, then we are waiting for some data to be read
	jr nz,MSDCallBack_CSW
	push de
	ld hl,USBDriverBuf
	ld b,0
	ld c,e
	ld de,(DataInAddr)
	ldir
	ld (DataInAddr),de
	;Calculate if any data left?
	pop de
	ld d,0
	ld hl,(DataInExpected)
	or a
	sbc hl,de
	jr z,MSDCallBack_OK
	jr c,MSDCallBack_OK	;though really not
	ld (DataInExpected),hl
	ld b,1
	U_CALL ReqData
	ret
MSDCallBack_OK:
	ld a,2
	ld (BOTState),a
	ld b,1
	U_CALL ReqData
	ret
MSDCallBack_CSW:
	;CSW
	;Perhaps later implement some checking...we can ignore it for now
	;follow chart on page 15 of the BOT specification if needed
	xor a
	ld (BOTState),a
	ret
