;UFI routines
;
;UFI_getName
;UFI_getCapacity
;UFI_Read
;UFI_Write
;UFI_Initialize
UFI_Initialize:
	;Initializes USB device for file system operations
	;Input - DE is a buffer where to store temporary read data (512 bytes)
	push de
	ld (SectorBuffer),de
	call UFI_getCapacity		;sets up (BPS)
	ld hl,(BPS)
	ld de,512
	or a
	sbc hl,de
	ld a,errLargeSect
	jp nz,displayErr
	ld hl,0
	ld (LBA),hl
	ld (LBA+2),hl
	pop de
	push de
	call UFI_Read
	ld b,4
	pop de
	ld hl,1C2h
	add hl,de
UFI_PartitionTable:
	ld a,(hl)
	cp 04h
	jr z,UFI_PartitionTable_OK		;FAT16 (<32 MB)
	cp 06h
	jr z,UFI_PartitionTable_OK		;FAT16 (32 MB - 2 GB)
	ld de,10h
	add hl,de
	djnz UFI_PartitionTable
	ld a,errNoParts
	jp displayErr
UFI_PartitionTable_OK:
	;In LBA mode, relative sectors is the starting LBA block
	ld bc,4
	add hl,bc
	;Relative sectors
	ld de,PartStart
	ldir
	ret
UFI_Write:
	;Input - (LBA) contains the starting address
	;Input - DE is the location to read the data to be written from
	;Output - Writes one block
	;C if error
	ld (DataInAddr),de		;AHA! WE SHALL FOOL YOU - IT'S REALLY DATAOUTADDR!
	ld de,(BPS)
	ld (DataInExpected),de	;IN SOVIET RUSSIA, Z80 FOOLS PROGRAMMER.
	ld hl,PacketData
	push hl
	ld (hl),9
	inc hl
	ld (hl),2Ah
	inc hl
	ld (hl),0
	inc hl
	ld a,(LBA+3)
	ld (hl),a
	inc hl
	ld a,(LBA+2)
	ld (hl),a
	inc hl
	ld a,(LBA+1)
	ld (hl),a
	inc hl
	ld a,(LBA)
	ld (hl),a
	inc hl
	ld (hl),0
	inc hl
	ld (hl),0
	inc hl
	ld (hl),1
	pop hl
	jp sendCmdOut
UFI_Read:
	;Input - (LBA) contains the starting address
	;Input - DE is the location to store the block of data
	;Output - Reads one block
	;C if error
	ld (DataInAddr),de
	ld de,(BPS)
	ld (DataInExpected),de
	ld hl,PacketData
	push hl
	ld (hl),9
	inc hl
	ld (hl),28h
	inc hl
	ld (hl),0
	inc hl
	ld a,(LBA+3)
	ld (hl),a
	inc hl
	ld a,(LBA+2)
	ld (hl),a
	inc hl
	ld a,(LBA+1)
	ld (hl),a
	inc hl
	ld a,(LBA)
	ld (hl),a
	inc hl
	ld (hl),0
	inc hl
	ld (hl),0
	inc hl
	ld (hl),1
	pop hl
	jp sendCmdIn
UFI_getCapacity:
	;Input - Nothing
	;Output - OP1 contains the size of the formatted medium in bytes
	;C returned if error
	ld hl,PacketData
	ld (DataInAddr),hl
	ld de,8
	ld (DataInExpected),de
	ld hl,UFI_getCapacity_UFI
	call sendCmdIn
	ret c
	ld ix,PacketData
	ld h,(ix)
	ld l,(ix+1)
	push ix
	B_CALL SetXXXXOP2
	ld hl,UFI_getCapacity_64k
	rst 20h
	B_CALL FPMult
	pop ix
	ld h,(ix+2)
	ld l,(ix+3)
	B_CALL SetXXXXOP2
	B_CALL FPAdd
	ld ix,PacketData+6
	ld h,(ix)
	ld l,(ix+1)
	ld (BPS),hl
	B_CALL SetXXXXOP2
	B_CALL FPMult
	or a
	ret
UFI_getCapacity_UFI:
	DB 6,25h,0,0,0,8,0
UFI_getCapacity_64k:
	DB 00,84h,65h,53h,60h,0,0,0,0
UFI_getName:
	;Input - Nothing (a properly initialized mass storage device)
	;Output - DE points to a vendor identification string, null-terminated
	;Output - HL points to a product identification string, null-terminated
	;C returned if error
	ld hl,PacketData
	ld (DataInAddr),hl
	ld de,36
	ld (DataInExpected),de
	ld hl,UFI_getName_UFI
	call sendCmdIn
	ret c
	ld hl,PacketData+8
	rst 20h
	xor a
	ld (OP1+8),a
	ld hl,PacketData+16
	ld de,OP2
	ld bc,16
	ldir
	ld (de),a
	ld hl,OP2
	ld de,OP1
	or a
	ret
UFI_getName_UFI:
	DB 6,12h,0,0,0,24h,0
